@@ -1,43 +1,26 @@
-# This file is part of QuTiP: Quantum Toolbox in Python.
-#
-#    Copyright (c) 2011 and later, The QuTiP Project.
-#    All rights reserved.
-#
-#    Redistribution and use in source and binary forms, with or without
-#    modification, are permitted provided that the following conditions are
-#    met:
-#
-#    1. Redistributions of source code must retain the above copyright notice,
-#       this list of conditions and the following disclaimer.
-#
-#    2. Redistributions in binary form must reproduce the above copyright
-#       notice, this list of conditions and the following disclaimer in the
-#       documentation and/or other materials provided with the distribution.
-#
-#    3. Neither the name of the QuTiP: Quantum Toolbox in Python nor the names
-#       of its contributors may be used to endorse or promote products derived
-#       from this software without specific prior written permission.
-#
-#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-#    PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-#    HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-#    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-#    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-#    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-#    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-#    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-#    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-###############################################################################
-import numpy as np
-import operator
-from scipy.sparse import (_sparsetools, isspmatrix, isspmatrix_csr,
-                          csr_matrix, coo_matrix, csc_matrix, dia_matrix)
-from scipy.sparse.sputils import (upcast, upcast_char, to_native, isdense, isshape,
-                      getdtype, isscalarlike, get_index_dtype)
-from scipy.sparse.base import spmatrix, isspmatrix, SparseEfficiencyWarning
 from warnings import warn
+import operator
+
+import numpy as np
+from scipy.sparse import (
+    csr_matrix, dia_matrix, isspmatrix, SparseEfficiencyWarning,
+)
+
+# fast_csr_matrix extends the internals of csr_matrix, and we need to
+# import parts of the internals of scipy.sparse to do that:
+import scipy.sparse
+import scipy.sparse._sparsetools as _sparsetools
+if hasattr(scipy.sparse, "_sputils"):
+    # SciPy 1.8.0 deprecated the public scipy.sparse.sputils interface and
+    # moved it to _sputils
+    from scipy.sparse._sputils import (
+        isdense, isscalarlike, upcast, get_index_dtype,
+    )
+else:
+    from scipy.sparse.sputils import (
+        isdense, isscalarlike, upcast, get_index_dtype,
+    )
+
 
 class fast_csr_matrix(csr_matrix):
     """
