@@ -139,7 +139,7 @@ def mesolve(H, rho0, tlist, c_ops=None, e_ops=None, args=None, options=None,
         dictionary of parameters for time-dependent Hamiltonians and
         collapse operators.
 
-    options : None / :class:`qutip.Options`
+    options : None / :class:`qutip.solver.Options`
         with options for the solver.
 
     progress_bar : None / BaseProgressBar
@@ -148,9 +148,9 @@ def mesolve(H, rho0, tlist, c_ops=None, e_ops=None, args=None, options=None,
 
     Returns
     -------
-    result: :class:`qutip.Result`
+    result: :class:`qutip.solver.Result`
 
-        An instance of the class :class:`qutip.Result`, which contains
+        An instance of the class :class:`qutip.solver.Result`, which contains
         either an *array* `result.expect` of expectation values for the times
         specified by `tlist`, or an *array* `result.states` of state vectors or
         density matrices corresponding to the times in `tlist` [if `e_ops` is
