@@ -1,35 +1,3 @@
-# This file is part of QuTiP: Quantum Toolbox in Python.
-#
-#    Copyright (c) 2011 and later, Paul D. Nation and Robert J. Johansson.
-#    All rights reserved.
-#
-#    Redistribution and use in source and binary forms, with or without
-#    modification, are permitted provided that the following conditions are
-#    met:
-#
-#    1. Redistributions of source code must retain the above copyright notice,
-#       this list of conditions and the following disclaimer.
-#
-#    2. Redistributions in binary form must reproduce the above copyright
-#       notice, this list of conditions and the following disclaimer in the
-#       documentation and/or other materials provided with the distribution.
-#
-#    3. Neither the name of the QuTiP: Quantum Toolbox in Python nor the names
-#       of its contributors may be used to endorse or promote products derived
-#       from this software without specific prior written permission.
-#
-#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-#    PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-#    HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-#    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-#    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-#    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-#    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-#    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-#    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-###############################################################################
 """
 This module contains settings for the QuTiP graphics, multiprocessing, and
 tidyup functionality, etc.
@@ -59,7 +27,7 @@ has_openmp = False
 # debug mode for development
 debug = False
 # Running on mac with openblas make eigh unsafe
-eigh_unsafe = False 
+eigh_unsafe = False
 # are we in IPython? Note that this cannot be
 # set by the RC file.
 ipython = False
@@ -148,19 +116,8 @@ def reset():
     for key in __default:
         __self[key] = __default[key]
 
-    import os
-    if 'QUTIP_NUM_PROCESSES' in os.environ:
-        num_cpus = int(os.environ['QUTIP_NUM_PROCESSES'])
-    elif 'cpus' in info:
-        import qutip.hardware_info
-        info = qutip.hardware_info.hardware_info()
-        num_cpus = info['cpus']
-    else:
-        try:
-            num_cpus = multiprocessing.cpu_count()
-        except:
-            num_cpus = 1
-    __self["num_cpus"] = num_cpus
+    from qutip.utilities import available_cpu_count
+    __self["num_cpus"] = available_cpu_count()
 
     try:
         from qutip.cy.openmp.parfuncs import spmv_csr_openmp
