@@ -1,37 +1,3 @@
-# This file is part of QuTiP: Quantum Toolbox in Python.
-#
-#    Copyright (c) 2011 and later, Paul D. Nation and Robert J. Johansson.
-#    All rights reserved.
-#
-#    Redistribution and use in source and binary forms, with or without
-#    modification, are permitted provided that the following conditions are
-#    met:
-#
-#    1. Redistributions of source code must retain the above copyright notice,
-#       this list of conditions and the following disclaimer.
-#
-#    2. Redistributions in binary form must reproduce the above copyright
-#       notice, this list of conditions and the following disclaimer in the
-#       documentation and/or other materials provided with the distribution.
-#
-#    3. Neither the name of the QuTiP: Quantum Toolbox in Python nor the names
-#       of its contributors may be used to endorse or promote products derived
-#       from this software without specific prior written permission.
-#
-#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-#    PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-#    HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-#    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-#    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-#    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-#    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-#    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-#    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-###############################################################################
-
-
 import pytest
 import numpy as np
 from pathlib import Path
@@ -42,7 +8,8 @@ from qutip.qip.circuit import (
     QubitCircuit, CircuitSimulator, Gate, Measurement, _ctrl_gates,
     _single_qubit_gates, _swap_like, _toffoli_like, _fredkin_like, _para_gates)
 from qutip import (tensor, Qobj, ptrace, rand_ket, fock_dm, basis,
-                   rand_dm, bell_state, ket2dm)
+                   rand_unitary_haar, bell_state, ket2dm, fidelity,
+                   average_gate_fidelity)
 from qutip.qip.qasm import read_qasm
 from qutip.qip.operations.gates import gate_sequence_product
 
@@ -257,11 +224,66 @@ class TestQubitCircuit:
             # Single control
             pytest.raises(ValueError, qc.add_gate, gate, [0], [1])
 
+        dummy_gate1 = Gate("DUMMY1")
+        inds = [1, 3, 4, 6]
+        qc.add_gate(dummy_gate1, index=inds)
+
+        # Test adding gates at multiple (sorted) indices at once.
+        # NOTE: Every insertion shifts the indices in the original list of
+        #       gates by an additional position to the right.
+        expected_gate_names = [
+            'CNOT',     # 0
+            'DUMMY1',   # 1
+            'SWAP',     # 2
+            'TOFFOLI',  # 3
+            'DUMMY1',   # 4
+            'SWAP',     # 5
+            'DUMMY1',   # 6
+            'SNOT',     # 7
+            'RY',       # 8
+            'DUMMY1',   # 9
+            'RY',       # 10
+        ]
+        actual_gate_names = [gate.name for gate in qc.gates]
+        assert actual_gate_names == expected_gate_names
+
+        dummy_gate2 = Gate("DUMMY2")
+        inds = [11, 0]
+        qc.add_gate(dummy_gate2, index=inds)
+
+        # Test adding gates at multiple (unsorted) indices at once.
+        expected_gate_names = [
+            'DUMMY2',   # 0
+            'CNOT',     # 1
+            'DUMMY1',   # 2
+            'SWAP',     # 3
+            'TOFFOLI',  # 4
+            'DUMMY1',   # 5
+            'SWAP',     # 6
+            'DUMMY1',   # 7
+            'SNOT',     # 8
+            'RY',       # 9
+            'DUMMY1',   # 10
+            'RY',       # 11
+            'DUMMY2',   # 12
+        ]
+        actual_gate_names = [gate.name for gate in qc.gates]
+        assert actual_gate_names == expected_gate_names
+
     def test_add_circuit(self):
         """
         Addition of a circuit to a `QubitCircuit`
         """
+
+        def customer_gate1(arg_values):
+            mat = np.zeros((4, 4), dtype=np.complex128)
+            mat[0, 0] = mat[1, 1] = 1.
+            mat[2:4, 2:4] = gates.rx(arg_values)
+            return Qobj(mat, dims=[[2, 2], [2, 2]])
+
         qc = QubitCircuit(6)
+        qc.user_gates = {"CTRLRX": customer_gate1}
+
         qc.add_gate("CNOT", targets=[1], controls=[0])
         test_gate = Gate("SWAP", targets=[1, 4])
         qc.add_gate(test_gate)
@@ -270,6 +292,7 @@ class TestQubitCircuit:
         qc.add_gate(test_gate, index=[3])
         qc.add_measurement("M0", targets=[0], classical_store=[1])
         qc.add_1q_gate("RY", start=4, end=5, arg_value=1.570796)
+        qc.add_gate("CTRLRX", targets=[1, 2], arg_value=np.pi/2)
 
         qc1 = QubitCircuit(6)
 
@@ -278,6 +301,9 @@ class TestQubitCircuit:
         # Test if all gates and measurements are added
         assert len(qc1.gates) == len(qc.gates)
 
+        # Test if the definitions of user gates are added
+        assert qc1.user_gates == qc.user_gates
+
         for i in range(len(qc1.gates)):
             assert (qc1.gates[i].name
                     == qc.gates[i].name)
@@ -313,6 +339,10 @@ class TestQubitCircuit:
                 assert (qc2.gates[i].controls[0]
                         == qc.gates[i].controls[0]+2)
 
+        # Test exception when the operators to be added are not gates or measurements
+        qc.gates[-1] = 0
+        pytest.raises(TypeError, qc2.add_circuit, qc)
+
     def test_add_state(self):
         """
         Addition of input and output states to a circuit.
@@ -495,7 +525,7 @@ class TestQubitCircuit:
         """
         Test for circuit with N-level system.
         """
-        mat3 = rand_dm(3, density=1.)
+        mat3 = rand_unitary_haar(3)
 
         def controlled_mat3(arg_value):
             """
@@ -510,7 +540,13 @@ class TestQubitCircuit:
         qc.user_gates = {"CTRLMAT3": controlled_mat3}
         qc.add_gate("CTRLMAT3", targets=[1, 0], arg_value=1)
         props = qc.propagators()
-        np.testing.assert_allclose(mat3, ptrace(props[0], 0) - 1)
+        final_fid = average_gate_fidelity(mat3, ptrace(props[0], 0) - 1)
+        assert pytest.approx(final_fid, 1.0e-6) == 1
+
+        init_state = basis([3, 2], [0, 1])
+        result = qc.run(init_state)
+        final_fid = fidelity(result, props[0] * init_state)
+        assert pytest.approx(final_fid, 1.0e-6) == 1.
 
     @pytest.mark.repeat(10)
     def test_run_teleportation(self):
@@ -529,7 +565,6 @@ class TestQubitCircuit:
 
         teleportation_sim_results = teleportation_sim.run(state)
         state_final = teleportation_sim_results.get_final_states(0)
-        probability = teleportation_sim_results.get_probabilities(0)
 
         final_measurement = Measurement("start", targets=[2])
         _, final_probabilities = final_measurement.measurement_comp_basis(state_final)
@@ -618,9 +653,6 @@ class TestQubitCircuit:
         qc = read_qasm(filepath)
 
         rand_state = rand_ket(2)
-        wstate = (tensor(basis(2, 0), basis(2, 0), basis(2, 1))
-                  + tensor(basis(2, 0), basis(2, 1), basis(2, 0))
-                  + tensor(basis(2, 1), basis(2, 0), basis(2, 0))).unit()
 
         state = tensor(tensor(basis(2, 0), basis(2, 0), basis(2, 0)),
                        rand_state)
@@ -641,6 +673,16 @@ class TestQubitCircuit:
                 np.testing.assert_allclose(probs_initial, probs_final)
                 assert sum(result_cbits[i]) == 1
 
+    def test_latex_code(self):
+        qc = QubitCircuit(1, num_cbits=1, reverse_states=True)
+        qc.add_measurement("M0", targets=0, classical_store=0)
+        exp = \
+            ' &  &  \\qw \\cwx[1]  & \\qw \\\\ \n &  &  \\meter & \\qw \\\\ \n'
+        assert qc.latex_code() == exp
 
-if __name__ == "__main__":
-    run_module_suite()
+    def test_latex_code_non_reversed(self):
+        qc = QubitCircuit(1, num_cbits=1, reverse_states=False)
+        qc.add_measurement("M0", targets=0, classical_store=0)
+        exp = ' &  &  \\meter & \\qw \\\\ \n &  ' + \
+              '&  \\qw \\cwx[-1]  & \\qw \\\\ \n'
+        assert qc.latex_code() == exp
